# Generated by Django 3.1.6 on 2021-02-10 13:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('contractor', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='contract',
            name='webhook_active',
            field=models.BooleanField(default=False),
        ),
    ]



from django.db.models import F
from django.utils import timezone

from celery import shared_task

from .models import Contract
from .utils import (
    download_and_save_file, resource_to_filename,
    get_integrity_for_resource
)


@shared_task(name='fetch_contract_result')
def fetch_contract_result(contract_id):
    try:
        contract = Contract.objects.get(id=contract_id)
    except Contract.DoesNotExist:
        return

    Contract.objects.filter(id=contract_id).update(
        version=F('version') + 1,
        updated=timezone.now()
    )
    contract.version += 1

    for filename, url in contract.get_source_files():
        path = contract.get_file_path(filename)
        download_and_save_file(url, path)

    apply_subresource_integrity.delay(contract.pk)


@shared_task(name='apply_subresource_integrity')
def apply_subresource_integrity(contract_id):
    try:
        contract = Contract.objects.get(id=contract_id)
    except Contract.DoesNotExist:
        return

    lines = []
    for resource in contract.get_resources():
        if resource.url.endswith(('.js', '.css')):
            integrity = get_integrity_for_resource(resource)
            resource.attributes['integrity'] = integrity
        line = resource_to_filename(resource)
        lines.append(line)

    contract.files = '\n'.join(lines)
    contract.save()



import base64
import hashlib
import logging
from urllib.parse import urlunparse, urlparse, urlencode

from django.core.files.storage import default_storage
from django.core.files.base import ContentFile

import lxml.html
import requests


logger = logging.getLogger(__name__)


def extract_html(contract, xpath):
    if '#' not in xpath:
        filename = 'index.html'
    else:
        filename, xpath = xpath.split('#', 1)

    path = contract.get_file_path(filename)
    try:
        content = default_storage.open(path).read()
        root = lxml.html.fromstring(content.decode('utf-8'))
    except Exception as e:
        logger.exception(e)
        return ''

    # Convert relative img paths to full media URLs
    selections = root.xpath(xpath)
    for selection in selections:
        for img in selection.xpath('//img'):
            if 'src' not in img.attrib:
                continue
            src = img.attrib['src']
            if not src.startswith(('http://', 'https://', '/')):
                img.attrib['src'] = contract.get_file_url(src)

    try:
        return '\n'.join(
            lxml.html.tostring(s, pretty_print=True).decode('utf-8')
            for s in selections
        )
    except Exception as e:
        logger.exception(e)
    return ''


def is_full_url(line):
    return line.startswith(('https://', 'http://'))


def download_and_save_file(url, path):
    response = requests.get(url)
    # FIXME: returned path is not stored, assumed to be the same
    default_storage.save(path, ContentFile(response.content))


def get_integrity_for_resource(resource):
    if resource.path is None:
        response = requests.get(resource.url)
        raw_bytes = response.content
    else:
        resource_file = default_storage.open(resource.path)
        raw_bytes = resource_file.read()
    raw_digest = hashlib.sha384(raw_bytes).digest()
    b64_digest = base64.b64encode(raw_digest).decode('ascii')
    return 'sha384-{}'.format(b64_digest)


def resource_to_filename(resource):
    parsed = urlparse(resource.url)
    query_params = {
        k: v if isinstance(v, str) else ''
        for k, v in resource.attributes.items()
    }
    query = urlencode(query_params, doseq=True)
    if resource.path:
        return '{}?{}'.format(resource.filename, query)
    return urlunparse(parsed._replace(query=query))



from django.urls import path

from .views import webhook, redirect_current

app_name = 'contractor'

urlpatterns = [
    path('webhook/<uuid:token>/', webhook, name='webhook'),
    path('file/<slug:slug>/current/<path:path>', redirect_current,
         name='redirect_current'),
]
